/*
    pwl9x "Windows 9x PassWord List reader"
    Copyright (C) 2001 Knispel Guillaume

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef   defs_h
#define   defs_h


#define ECMDLINE 1 /* incorrect command line */
#define EPWD     2 /* bad / missing password */
#define EPWL     3 /* bad / missing pwl file */
#define EMEM     4 /* not enough memory */
#define EMET     5 /* methode not available */

typedef enum {
  true=1,
  false=0
} bool;

typedef unsigned char      byte;
typedef unsigned short     word;
typedef unsigned long      dword;
typedef unsigned long long qword;

typedef byte hres[16];
typedef byte hblk[64];

typedef struct {
  dword magic;             /* 968582E3h 4 OSR2 encoding */
  dword unknown_d;         /* Strange counter */
  byte unusedlinks[0x100]; /* Ressources index */
  byte unusedkeys[0x100];  /* Ressources keys */
  dword signcheckOffs;     /* 252h 4 now */
  dword salt[0x11];        /* Lot of salt (encoding by sections) */
  word unknown_w;          /* Maybe just alignement */
  hres sign;               /* Signature, xor crypted, and to be hashed */
  hres check;              /* Check value of last hash */
  word sectionptr[0xF];    /* Byte ptr 2 sections */
} Header;

typedef union {
  byte *stream;
  Header *head;
} Pwl;

typedef struct {
  bool have_action, do_list, do_head, do_crack;
  bool have_methode;
  char *methode;
  bool have_dico;
  char *dico_file;
  bool have_alphabet;
  char *abc_file;
  bool have_pwd;
  char *pwd;
  bool have_username;
  char *username;
  bool have_sysini;
  char *sysini_file;
  char *pwl_file;
  dword size;
} Params;

typedef struct {        /* main cryptotable, produced after hashing */
  byte body[256];                              /* data table itself */
  byte c1,c2;                    /* crypto pointers into data table */
} xor_table;

typedef struct {                   /* main hash container structure */
 hblk buf;                               /* buffer for hashing data */
 dword cnt[4];                            /* internal hash counters */
 dword idx[2];                    /* internal data position pointer */
} hash_container;


#endif /* defs_h */
