/*
    pwl9x "Windows 9x PassWord List reader"
    Copyright (C) 2001 Knispel Guillaume

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

#include "defs.h"
#include "cpu.h"
#include "mmx.h"
#include "list.h"

int load_raw_pwl(Params *p, Pwl *pwl)
{
  FILE *fp; int a;

  if (!(fp = fopen(p->pwl_file, "rb"))) return 0;
  fseek(fp, 0, SEEK_END); /* this way we make next ftell return the file size */
  if (!(pwl->stream = malloc(p->size = ftell(fp)))) { fprintf(stderr, "Could not allocate enough memory.\n"); exit(EMEM); }
  rewind(fp);

  a = fread(pwl->stream, p->size, 1, fp);
  fclose(fp);
  if (!a || pwl->head->magic != 0x968582E3 || pwl->head->signcheckOffs != 0x252) { free(pwl->stream); return 0; }
  return 1;
}

void try_extract_username(Params *p, Pwl *pwl)
{
#define LINESIZE 1024
  char line[LINESIZE], *fnp, *v;
  FILE *s = fopen(p->sysini_file, "rb");

  for(fnp = p->pwl_file; *fnp; fnp++); for(; fnp > p->pwl_file && *fnp != '/'; fnp--); if (*fnp == '/') fnp++;
  if (!s) {
    fprintf(stderr, "Unable to open the sysini windows file !\n");
    return;
  }
  while((v = fgets(line, LINESIZE, s))) {
    if (!strncmp("[Password Lists]", line, strlen("[Password Lists]"))) break;
  }
  if (!v) {
    fprintf(stderr, "Unable to find the %s section !\n", "[Password Lists]");
    return;
  }
  while((v = fgets(line, LINESIZE, s))) {
    for(; *v; v++); for(; v > line && *v != '\\'; v--); if (*v == '\\') v++;
    if (!strncmp(fnp, v, strlen(fnp))) {
      v = strchr(line, '=');
      if (!v) return;
      *v = 0;
      if (!(p->username = malloc(strlen(line)+1))) { fprintf(stderr, "Could not allocate enough memory.\n"); exit(EMEM); }
      strcpy(p->username, line);
      p->have_username = true;
      return;
    }
    if (v[0] == '[') {
      fprintf(stderr, "No name match !\n");
      return;
    }
  }

  return;
}

void disp_head(Params *p, Pwl *pwl)
{
  int i;
  printf("Header\n---\n\n");

  printf("Variable\tContent\n");

  printf("Magic\t\t0x%X\n", (int) pwl->head->magic);
  printf("Unknown C\t0x%X\n", (int) pwl->head->unknown_d);
  printf("Links\t\t"); for (i=0; i < 0x100; i++) printf("0x%X\n\t\t", (int) pwl->head->unusedlinks[i]); printf("\n");
  printf("Keys\t\t");  for (i=0; i < 0x100; i++) printf("0x%X\n\t\t", (int) pwl->head->unusedkeys[i]); printf("\n");
  printf("ChkPwdOff\t0x%X\n", (int) pwl->head->signcheckOffs);
  printf("Salts\t\t"); for (i=0; i < 0x11; i++) printf("0x%X\n\t\t", (int) pwl->head->salt[i]); printf("\n");
  printf("Unknown A\t0x%X\n\n", (int) pwl->head->unknown_w);
}

void help()
{
  fprintf(stderr, "Usage\n---\n\n"\
	          "pwl9x action [options] pwlfile.pwl\n"\
	          "action : [-l] [-t] [-c]\n"\
	          "\t-l : list ressources of the given pwl file\n"\
	          "\t-t : display the header\n"\
	          "\t-c : crack (test lots of pwd)\n"\
	          "options : [-p PASSWORD] [-m methode] [-d dictionary] [-a alphabet] [-u USERNAME]\n          [-s system.ini]\n\n");
}

int main(int argc, char *argv[])
{
  Params p;
  Pwl pwl;
  int opt = 0;

  memset(&p, 0, sizeof(p));

  fprintf(stderr, "Windows 9x PassWord List reader\n"\
	          "[pwl9x v0.04] (C) 2001 Xilun.\n\n");

  while (opt != '?' && (opt = getopt(argc, argv, "ltcp:m:d:a:u:s:")) != EOF) {
    switch (opt) {
    case 'l' :
      p.have_action = true; p.do_list = true;
      break;
    case 't' :
      p.have_action = true; p.do_head = true;
      break;
    case 'c' :
      p.have_action = true; p.do_crack = true;
      break;
    case 'p' :
      p.have_pwd = true; p.pwd = optarg;
      break;
    case 'm' :
      p.have_methode = true; p.methode = optarg;
      break;
    case 'u' :
      p.have_username = true; p.username = optarg;
      break;
    case 'd' :
      p.have_dico = true; p.dico_file = optarg;
      break;
    case 'a' :
      p.have_alphabet = true; p.abc_file = optarg;
      break;
    case 's' :
      p.have_sysini = true; p.sysini_file = optarg;
      break;
    case '?' :
      help();
      exit(ECMDLINE);
      break;
    default  :
      fprintf(stderr, "Error in getopt output parsing\n");
      exit(ECMDLINE);
      break;
    }
  }

  if (!p.have_action || !argv[optind] || argv[optind+1]) { help(); exit(ECMDLINE); }
  else p.pwl_file = argv[optind];
  
  if (!load_raw_pwl(&p, &pwl)) { fprintf(stderr, "PWL file not loaded or bad PWL file, giving up.\n"); exit(EPWL); }
  if (p.have_sysini) try_extract_username(&p, &pwl); /* the username in system.ini file has precedence over the command line provided username */
  if (p.do_head) disp_head(&p, &pwl);
  if (p.do_list || p.do_crack) {
    if (!p.have_username) { fprintf(stderr, "Username not found, giving up.\n"); exit(ECMDLINE); }
    if (p.do_crack) { chose_crack_core(); try_crack_pwl(&p, &pwl); }
    if (p.do_list) {
      if (!p.have_pwd) { fprintf(stderr, "Password unknown, giving up.\n"); exit(EPWD); }
      try_list(&p, &pwl);
    }
  }

  return 0;
}
